@@ -1,10 +1,24 @@
-//! Backend-neutral Mime database implementation.
+// SPDX-FileCopyrightText: 2021 Lynnesbian
+// SPDX-License-Identifier: LGPL-3.0-or-later
+
+//! Backend-neutral Mime database abstraction.
 
 use cfg_if::cfg_if;
 use mime::Mime;
 
+/// A thin wrapper around either [`Infer`] or [`xdg-mime::SharedMimeInfo`], depending on which [cargo features]
+///  fif was compiled with. By default, fif uses an [`Infer`]-based implementation on Windows, and an
+/// [`xdg-mime`]-based one everywhere else. This behaviour can be changed at compile time by using the aforementioned
+/// [cargo features].
+///
+/// [cargo features]: https://gitlab.com/Lynnesbian/fif/-/wikis/Cargo-Features
+/// [`Infer`]: https://docs.rs/infer/
+/// [`xdg-mime::SharedMimeInfo`]: https://docs.rs/xdg-mime/0/xdg_mime/struct.SharedMimeInfo.html
+/// [`xdg-mime`]: https://docs.rs/xdg-mime/
 pub trait MimeDb {
+	/// Initialise the database.
 	fn init() -> Self;
+	/// Given a slice of bytes, returns the inferred MIME type, if any.
 	fn get_type(&self, data: &[u8]) -> Option<Mime>;
 }
 
@@ -12,6 +26,7 @@ cfg_if! {
 	if #[cfg(any(all(unix, feature = "infer-backend"), all(not(unix), not(feature = "xdg-mime-backend"))))] {
 		use std::str::FromStr;
 
+		/// The [`Infer`](https://docs.rs/infer/)-based implementation of [`MimeDb`].
 		pub struct InferDb {
 			db: infer::Infer,
 		}
@@ -43,9 +58,21 @@ cfg_if! {
 					buf.len() >= 34 && buf.starts_with(b"RPA-") && buf[7] == b' ' && buf[24] ==b' '
 				});
 
+				info.add("audio/x-aiff", "aiff", |buf| {
+					// as added by https://github.com/bojand/infer/pull/48/files
+					// this should be removed when (if) that PR is accepted
+					buf.starts_with(b"\x46\x4f\x52\x4d\x41\x49\x46\x46")
+				});
+
+				// info.add("application/x-msi", "msi", |buf| {
+				// TODO: find a way to detect MSI files properly - this just detects those weird windows OLE files and therefore
+				// also picks up on .doc files
+				// 	buf.starts_with(b"\xd0\xcf\x11\xe0\xa1\xb1\x1a\xe1")
+				// });
+
 				info.add("image/svg+xml", "svg", |buf| {
-					// before doing the moderately expensive SVG check, we should make sure that the input is actually SGML-ish
-					// by "SGML-ish", i mean starts with anywhere from zero to ∞-1 whitespace characters, and then a less than sign,
+					// before doing the moderately expensive SVG check, we should make sure that the input is actually SGML-ish,
+					// by which i mean, starts with anywhere from zero to ∞-1 whitespace characters, and then a less than sign,
 					// and then there's some other stuff we don't care about right now
 
 					// so, here comes our fancy pants """""SGML-ish validator"""""
@@ -72,17 +99,20 @@ cfg_if! {
 						.any(|buf| identifiers.iter().any(|id| buf.starts_with(id)))
 				});
 
-				// unmut
-				let info = info;
-
 				Self { db: info }
 			}
 
 			fn get_type(&self, data: &[u8]) -> Option<Mime> {
-				self.db.get(data).map(|f| Mime::from_str(f.mime_type()).unwrap())
+				if let Some(mime) = self.db.get(data) {
+					match Mime::from_str(mime.mime_type()) {
+						Err(_) => None,
+						Ok(m) => Some(m),
+					}
+				} else { None }
 			}
 		}
 	} else {
+		/// The [`xdg-mime`](https://docs.rs/xdg-mime/)-based implementation of [`MimeDb`].
 		pub struct XdgDb {
 			db: xdg_mime::SharedMimeInfo,
 		}
