@@ -1,21 +1,30 @@
-//! [Clap] struct used to parse command line arguments.
+// SPDX-FileCopyrightText: 2021 Lynnesbian
+// SPDX-License-Identifier: LGPL-3.0-or-later
+
+//! Command line argument parsing logic and associated functions.
 
-use crate::string_type::String as StringType;
-use crate::utils::{clap_long_version, clap_version};
-use cfg_if::cfg_if;
-use clap::{AppSettings, Clap};
 use std::collections::BTreeSet;
 use std::path::PathBuf;
 
+use cfg_if::cfg_if;
+use clap::{Parser, ArgEnum};
+
+use crate::utils::{CLAP_LONG_VERSION, CLAP_VERSION};
+use crate::String as StringType;
+
 cfg_if! {
 	if #[cfg(windows)] {
+		/// The default [`OutputFormat`] to use.
 		const DEFAULT_FORMAT: &str = "powershell";
 	} else {
+		/// The default [`OutputFormat`] to use.
 		const DEFAULT_FORMAT: &str = "sh";
 	}
 }
 
-#[derive(Clap, PartialEq, Debug)]
+#[derive(ArgEnum, PartialEq, Debug, Copy, Clone)]
+/// The format to use when running fif without the `--fix` flag. Specified at runtime with the `-o`/`--output-format`
+/// flag.
 pub enum OutputFormat {
 	/// A Bourne shell compatible script.
 	#[clap(alias = "shell", alias = "bash")]
@@ -30,23 +39,47 @@ pub enum OutputFormat {
 	Json,
 }
 
-// TODO: convert this to macro style?: https://docs.rs/clap/3.0.0-beta.2/clap/index.html#using-macros
+#[derive(ArgEnum, PartialEq, Debug, Copy, Clone)]
+/// Specifies under what conditions the user should be prompted when running fif in `--fix` mode. Defaults to `Error`.
+/// Specified at runtime with the `-p`/`--prompt` flag.
+pub enum Prompt {
+	/// Never prompt.
+	Never,
+	/// Prompt only on errors, and on overwrites, if `--overwrite` is set.
+	Error,
+	/// Prompt for every rename.
+	Always,
+}
 
-#[derive(Clap, Debug)]
+#[derive(Parser, Debug)]
 #[allow(clippy::struct_excessive_bools)]
 #[clap(
-	version = clap_version(),
-	long_version = clap_long_version(),
+	version = CLAP_VERSION.as_str(),
+	long_version = CLAP_LONG_VERSION.as_str(),
 	author = option_env!("CARGO_PKG_AUTHORS").unwrap_or("Lynnesbian"),
 	about = option_env!("CARGO_PKG_DESCRIPTION").unwrap_or("File Info Fixer"),
 	before_help = "Copyright © 2021 Lynnesbian under the GPL3 (or later) License.",
-	before_long_help = "Copyright © 2021 Lynnesbian\n\
+	after_long_help = "Copyright © 2021 Lynnesbian\n\
 	This program is free software: you can redistribute it and/or modify \
 	it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 \
 	of the License, or (at your option) any later version.",
-	setting(AppSettings::ColoredHelp)
+	max_term_width = 120
 )]
+/// [`Clap`]-derived struct used to parse command line arguments.
 pub struct Parameters {
+	/// Automatically rename files to use the correct extension, prompting the user for every rename.
+	#[clap(long)]
+	pub fix: bool,
+
+	/// Requires --fix. Should fif prompt you `Never`, only on `Error`s and overwrites, or `Always`?
+	#[clap(short = 'p', long, arg_enum, requires = "fix")]
+	pub prompt: Option<Prompt>,
+
+	/// Requires --fix. Allow overwriting files. Warning: When used in combination with `--prompt never`, fif will
+	/// overwrite files without prompting!
+	#[clap(long, requires = "fix")]
+	pub overwrite: bool,
+
 	// NOTE: clap's comma-separated argument parser makes it impossible to specify extensions with commas in their name -
 	// `-e sil\,ly` is treated as ["sil", "ly"] rather than as ["silly"], no matter how i escape the comma (in bash,
 	// anyway). is this really an issue? it does technically exclude some perfectly valid extensions, but i've never seen
@@ -54,7 +87,7 @@ pub struct Parameters {
 	/// Only examine files with these extensions.
 	/// Multiple extensions can be specified by either using the flag multiple times (`-e jpg -e png -e gif`), or by
 	/// separating them with commas (`-e jpg,png,gif`).
-	#[clap(short, long, use_delimiter = true, require_delimiter = true, value_name = "ext", validator = lowercase_exts)]
+	#[clap(short, long, use_delimiter = true, require_delimiter = true, value_name = "ext", takes_value = true, validator = validate_exts)]
 	pub exts: Option<Vec<StringType>>,
 
 	/// Use these preset lists of extensions as the search filter (comma-separated list).
@@ -72,7 +105,8 @@ pub struct Parameters {
 
 	/// Don't scan files with these extensions.
 	/// This option takes precedence over extensions specified with `-e` or `-E`.
-	#[clap(short = 'x', long, use_delimiter = true, require_delimiter = true, value_name = "ext", validator = lowercase_exts)]
+	#[clap(short = 'x', long, use_delimiter = true, require_delimiter = true, value_name = "ext", validator =
+	validate_exts)]
 	pub exclude: Option<Vec<StringType>>,
 
 	/// Exclude files using a preset list of extensions.
@@ -128,10 +162,30 @@ pub struct Parameters {
 	/// The directory to process.
 	#[clap(name = "DIR", default_value = ".", parse(from_os_str))]
 	pub dir: PathBuf,
+
+	/// Don't rename files with extensions unknown to fif.
+	/// For example, with this option, fif will not rename "image.unknown" to "image.jpg"
+	#[clap(short = 'I', long)]
+	pub ignore_unknown_exts: bool,
+
+	#[cfg(feature = "multi-threaded")]
+	/// Number of jobs (threads) to use when scanning results.
+	/// The default behaviour is to use one thread per CPU thread. This behaviour can be manually requested by setting
+	/// `-j 0`. Using `-j 1` will disable multi-threading behaviour, as if you had compiled fif with the multi-threading
+	/// feature disabled. Setting more jobs than you have CPU threads is not recommended.
+	#[clap(short = 'j', long, default_value = "0")]
+	pub jobs: usize,
 }
 
-fn lowercase_exts(exts: &str) -> Result<(), String> {
+/// Validation function for argument parsing that ensures passed-in extensions are lowercase, and that the user
+/// didn't supply an empty list.
+fn validate_exts(exts: &str) -> Result<(), String> {
 	// TODO: i would much rather accept uppercase exts and convert them to lowercase than just rejecting lowercase exts...
+
+	if exts.is_empty() {
+		return Err(String::from("Cannot specify empty extensions"));
+	}
+
 	if exts.to_lowercase() != exts {
 		return Err(String::from("Supplied extensions must be lowercase"));
 	}
@@ -139,7 +193,8 @@ fn lowercase_exts(exts: &str) -> Result<(), String> {
 }
 
 /// Further options relating to scanning.
-#[derive(PartialEq, Debug)]
+#[derive(PartialEq, Debug, Copy, Clone)]
+#[allow(clippy::struct_excessive_bools)]
 pub struct ScanOpts {
 	/// Whether hidden files and directories should be scanned.
 	pub hidden: bool,
@@ -147,6 +202,8 @@ pub struct ScanOpts {
 	pub extensionless: bool,
 	/// Should symlinks be followed?
 	pub follow_symlinks: bool,
+	/// Whether to rename files with unknown extensions.
+	pub ignore_unknown_exts: bool,
 }
 
 impl Parameters {
@@ -209,18 +266,18 @@ impl Parameters {
 		}
 	}
 
+	/// Populates a [`ScanOpts`] struct with values from [`self`].
 	pub const fn get_scan_opts(&self) -> ScanOpts {
 		ScanOpts {
 			hidden: self.scan_hidden,
 			extensionless: self.scan_extensionless,
 			follow_symlinks: self.follow_symlinks,
+			ignore_unknown_exts: self.ignore_unknown_exts,
 		}
 	}
 
-	pub fn default_verbosity(&self) -> log::LevelFilter {
-		#![allow(clippy::missing_const_for_fn)]
-		// match was not permitted inside const functions until 1.46
-
+	/// Gets the verbosity set by the user via the `-v` and `-q` flags.
+	pub const fn get_verbosity(&self) -> log::LevelFilter {
 		use log::LevelFilter;
 
 		match self.quiet {
@@ -239,7 +296,7 @@ impl Parameters {
 }
 
 /// Sets of extensions for use with [Parameter](crate::parameters::Parameters)'s `-E` flag.
-#[derive(Clap, PartialEq, Debug)]
+#[derive(ArgEnum, PartialEq, Debug, Copy, Clone)]
 pub enum ExtensionSet {
 	/// Extensions used for image file formats, such as `png`, `jpeg`, `webp`, etc.
 	Images,
@@ -249,7 +306,7 @@ pub enum ExtensionSet {
 	#[clap(alias = "videos")]
 	Video,
 	/// Extensions used for media file formats. This acts as a combination of the [Images](ExtensionSet::Images),
-	/// [Audio](ExtensionSet::Audio) and [Videos](ExtensionSet::Videos) variants.
+	/// [Audio](ExtensionSet::Audio) and [Video](ExtensionSet::Video) variants.
 	Media,
 	/// Extensions used for document file formats, such as `pdf`, `odt`, `docx`, etc.
 	Documents,
@@ -268,12 +325,7 @@ impl ExtensionSet {
 			Self::Images => mime_guess::get_mime_extensions_str("image/*").unwrap().to_vec(),
 			Self::Audio => mime_guess::get_mime_extensions_str("audio/*").unwrap().to_vec(),
 			Self::Video => mime_guess::get_mime_extensions_str("video/*").unwrap().to_vec(),
-			Self::Media => [
-				Self::Images.extensions(),
-				Self::Audio.extensions(),
-				Self::Video.extensions(),
-			]
-			.concat(),
+			Self::Media => [Self::Images.extensions(), Self::Audio.extensions(), Self::Video.extensions()].concat(),
 			Self::Documents => vec![
 				"pdf", "doc", "docx", "ppt", "pptx", "xls", "xlsx", "csv", "tsv", "odt", "ods", "odp", "oda", "rtf", "ps",
 				"pages", "key", "numbers",
