@@ -6,7 +6,7 @@ use std::collections::{BTreeMap, HashMap};
 use std::ffi::OsStr;
 use std::path::{Path, PathBuf};
 
-use clap::Clap;
+use clap::Parser;
 use fif::files::{mime_extension_lookup, scan_directory, scan_from_walkdir, BUF_SIZE};
 use fif::findings::Findings;
 use fif::formats::{Format, PowerShell, Shell};
@@ -29,13 +29,13 @@ const ZIP_BYTES: &[u8] = b"PK\x03\x04";
 /// Ensure that `extension_from_path` successfully returns the extension from a set of paths.
 fn get_ext() {
 	let ext_checks: HashMap<_, Option<&OsStr>> = hashmap![
-	Path::new("test.txt") => Some(OsStr::new("txt")),
-	Path::new("test.zip") => Some(OsStr::new("zip")),
-	Path::new("test.tar.gz") => Some(OsStr::new("gz")),
-	Path::new("test.") => Some(OsStr::new("")),
-	Path::new("test") => None,
-	Path::new(".hidden") => None,
-		];
+		Path::new("test.txt") => Some(OsStr::new("txt")),
+		Path::new("test.zip") => Some(OsStr::new("zip")),
+		Path::new("test.tar.gz") => Some(OsStr::new("gz")),
+		Path::new("test.") => Some(OsStr::new("")),
+		Path::new("test") => None,
+		Path::new(".hidden") => None,
+	];
 
 	for (path, ext) in ext_checks {
 		assert_eq!(path.extension(), ext);
@@ -43,7 +43,7 @@ fn get_ext() {
 }
 
 #[test]
-/// Ensure that the mime types for JPEG, PNG, PDF, and ZIP are detected from their magic numbers.
+/// Ensure that the MIME types for JPEG, PNG, PDF, and ZIP are detected from their magic numbers.
 fn detect_type() {
 	assert_eq!(MIMEDB.get_type(JPEG_BYTES), Some(IMAGE_JPEG));
 	assert_eq!(MIMEDB.get_type(PNG_BYTES), Some(IMAGE_PNG));
@@ -55,23 +55,28 @@ fn detect_type() {
 /// Ensure that `mime_extension_lookup` works as expected, and that the set of extensions for JPEG, PNG, PDF, and ZIP
 /// contain "jpg", "png", "pdf", and "zip", respectively.
 fn recommend_ext() {
-	assert!(mime_extension_lookup(IMAGE_JPEG.essence_str().into())
-		.unwrap()
-		.contains(&String::from("jpg")));
-	assert!(mime_extension_lookup(IMAGE_PNG.essence_str().into())
-		.unwrap()
-		.contains(&String::from("png")));
-	assert!(mime_extension_lookup(APPLICATION_PDF.essence_str().into())
-		.unwrap()
-		.contains(&String::from("pdf")));
-	assert!(mime_extension_lookup(APPLICATION_ZIP.essence_str().into())
-		.unwrap()
-		.contains(&String::from("zip")));
+	let tests = hashmap![
+		&IMAGE_JPEG => "jpg",
+		&IMAGE_PNG => "png",
+		&APPLICATION_PDF => "pdf",
+		&*APPLICATION_ZIP => "zip",
+	];
+
+	for (mime, ext) in tests {
+		assert!(
+			mime_extension_lookup(mime.essence_str().into())
+				.unwrap()
+				.contains(&String::from(ext)),
+			"mime_extension_lookup for {} didn't contain {}!",
+			mime.essence_str(),
+			ext
+		);
+	}
 }
 
 #[test]
 /// Create a simple directory with some files, run `scan_directory` on it, and ensure that the files have their
-/// associated mime types correctly deduced.
+/// associated MIME types correctly deduced.
 fn simple_directory() {
 	use std::borrow::Borrow;
 	use std::env::set_current_dir;
@@ -136,19 +141,16 @@ fn simple_directory() {
 			// the only invalid file detected should be "wrong.jpg", which is a misnamed png file
 			// 1. ensure detected extension is "jpg"
 			assert_eq!(result.file.as_path().extension().unwrap(), OsStr::new("jpg"));
-			// 2. ensure detected mime type is IMAGE_PNG
+			// 2. ensure detected MIME type is IMAGE_PNG
 			assert_eq!(result.mime, IMAGE_PNG);
 			// 3. ensure the recommended extension for "wrong.jpg" is "png"
 			assert_eq!(&result.recommended_extension().unwrap(), &String::from("png"));
 			// 4. ensure the recommended filename for "wrong.jpg" is "wrong.png"
-			assert_eq!(
-				result.recommended_path().unwrap().file_name(),
-				Some(OsStr::new("wrong.png"))
-			);
+			assert_eq!(result.recommended_path().unwrap().file_name(), Some(OsStr::new("wrong.png")));
 			continue;
 		}
 
-		// check if the recommended extension for this file is in the list of known extensions for its mimetype - for
+		// check if the recommended extension for this file is in the list of known extensions for its MIME type - for
 		// example, if the file is determined to be an IMAGE_PNG, its recommended extension should be one of the extensions
 		// returned by `mime_extension_lookup(IMAGE_PNG)`.
 		assert!(mime_extension_lookup(result.mime.essence_str().into())
@@ -164,8 +166,8 @@ fn simple_directory() {
 			.to_string_lossy()
 			.starts_with("test"));
 
-		// make sure the guessed mimetype is correct based on the extension of the scanned file
-		// because we already know that the extensions match the mimetype (as we created these files ourselves earlier in
+		// make sure the guessed MIME type is correct based on the extension of the scanned file
+		// because we already know that the extensions match the MIME type (as we created these files ourselves earlier in
 		// the test), all files with the "jpg" extension should be IMAGE_JPEGs, etc.
 		let ext = result.file.as_path().extension().unwrap();
 		assert_eq!(
@@ -292,6 +294,7 @@ fn exclude_set_overrides_include_set() {
 #[test]
 /// Ensure that badly formed command line arguments are rejected.
 fn rejects_bad_args() {
+	use assert_cmd::Command;
 	let tests = [
 		// Non-existent flags:
 		vec!["fif", "-abcdefghijklmnopqrstuvwxyz"],
@@ -303,6 +306,8 @@ fn rejects_bad_args() {
 		vec!["fif", "-X", "pebis"],
 		// `-e` with nothing but commas:
 		vec!["fif", "-e", ",,,,,"],
+		// `-x` with nothing but commas:
+		vec!["fif", "-x", ",,,,,"],
 		// `-j` with a negative value:
 		vec!["fif", "-j", "-1"],
 		// `--prompt` without `--fix`:
@@ -312,10 +317,67 @@ fn rejects_bad_args() {
 	];
 
 	for test in &tests {
+		// first, try testing the flags against the Parameters struct...
 		assert!(Parameters::try_parse_from(test).is_err(), "Failed to reject {:?}", test);
+		// ...then, make sure it actually works against the binary
+		let mut cmd = Command::cargo_bin("fif").unwrap();
+		cmd.args(test).assert().failure();
 	}
 }
 
+#[test]
+/// Ensure that a few simple, well-formed command line argument cases pass
+fn accepts_good_args() {
+	use assert_cmd::Command;
+
+	// all of these commands pass either the version or help flag, ensuring that they won't fail for reasons relating
+	// to filesystem access
+	let tests = [
+		vec!["-V"],
+		vec!["--version"],
+		vec!["-E", "images", "--version"],
+		vec!["-h"],
+		vec!["--help"],
+		vec!["dir_name", "--version"],
+	];
+
+	for test in &tests {
+		let mut cmd = Command::cargo_bin("fif").unwrap();
+		cmd.args(test).assert().success();
+	}
+}
+
+#[test]
+/// Ensures that output from the `-V` and `--version` flags is formatted properly.
+fn check_version_output() {
+	use std::string::String;
+	use assert_cmd::Command;
+	use regex::Regex;
+
+	// test `-V` matches the format of "fif x.y.z"
+	let mut cmd = Command::cargo_bin("fif").unwrap();
+	let output = cmd.arg("-V").ok().unwrap().stdout;
+	let output = String::from_utf8(output).unwrap();
+	assert!(
+		Regex::new(r#"fif v([0-9]\.){2}[0-9]"#).unwrap().is_match(output.trim()),
+		"\"{}\" does not match the expected `-v` format!",
+		output
+	);
+
+
+	// test `--version` matches the format of "fif x.y.z (OS, example backend, commit #1234abc)"
+	let mut cmd = Command::cargo_bin("fif").unwrap();
+	let output = cmd.arg("--version").ok().unwrap().stdout;
+	let output = String::from_utf8(output).unwrap();
+	assert!(
+		Regex::new(r#"fif v([0-9]\.){2}[0-9] \(.+, .+ backend, (unknown commit|commit #[[:xdigit:]]{7})\)"#)
+			.unwrap()
+			.is_match(output.trim()),
+		"\"{}\" does not match the expected `--version` format!",
+		output.trim()
+	);
+}
+
 #[test]
 /// Generate random series of bytes and try to identify them. This test makes no assertions and can only fail if the
 /// mime database somehow panics or hangs.
@@ -403,7 +465,7 @@ fn test_json() {
 		.read_to_string(&mut contents)
 		.expect("Failed to read from cursor to string");
 
-	// the output should contain the file's mime type
+	// the output should contain the file's MIME type
 	assert!(
 		contents.contains(IMAGE_JPEG.essence_str()),
 		"JSON output doesn't contain move command!\n===\n{}",
@@ -430,15 +492,20 @@ fn media_contains_audio_video_images() {
 }
 
 #[test]
-/// Ensure that the `writables!` macro produces the output it should.
+/// Ensure that the `writables!` and `writablesln!` macros produce the output they should.
 fn writables_is_correct() {
 	use fif::formats::Writable;
-	use fif::writables;
+	use fif::{writables, writablesln};
 
 	assert_eq!(
 		&["henlo".into(), Path::new("henlo").into(), Writable::Newline,],
 		writables!["henlo", (Path::new("henlo")), Newline]
 	);
+
+	assert_eq!(
+		&["henlo".into(), Path::new("henlo").into(), Writable::Newline, Writable::Newline],
+		writablesln!["henlo", (Path::new("henlo")), Newline]
+	);
 }
 
 #[test]
@@ -470,10 +537,7 @@ fn verbosity() {
 #[test]
 /// Ensures `os_name()`'s output is the same as [`std::env::consts::OS`], capitalisation notwithstanding
 fn validate_os_name() {
-	assert_eq!(
-		fif::utils::os_name().to_lowercase(),
-		std::env::consts::OS.to_lowercase()
-	);
+	assert_eq!(fif::utils::os_name().to_lowercase(), std::env::consts::OS.to_lowercase());
 }
 
 #[test]
@@ -505,6 +569,7 @@ fn sort_findings() {
 }
 
 #[test]
+#[cfg(not(all(target_endian = "big", target_pointer_width = "32")))]
 /// Ensures that [`SmartString`]s don't deviate from std's Strings
 fn validate_string_type() {
 	use std::string::String as StdString;
@@ -516,6 +581,6 @@ fn validate_string_type() {
 		SmartString::from("A long and therefore heap-allocated string"),
 		StdString::from("A long and therefore heap-allocated string")
 	);
-	// uncomment if i ever update to smartstring >= 0.2.9
+
 	smartstring::validate();
 }
